@@ -17,7 +17,7 @@ use Regexp::Common qw/comment delimited/;
 our $WS_RE = qr/[ \t\v\n\f]/;          # C.f. doAsmOpaque()
 our $ASM_COMMENT_RE = qr/(?:;[^\n]*|$RE{comment}{'C++'})/;
 
-our $VERSION = '0.36'; # TRIAL VERSION
+our $VERSION = '0.40'; # TRIAL VERSION
 
 
 # ----------------------------------------------------------------------------------------
@@ -32,9 +32,8 @@ sub new {
   map {$logInfo{$_}++} @{$logInfo};
 
   my $grammarName = $opts{grammarName} || 'ISO-ANSI-C-2011';
-  my $start = $opts{start} || '';
 
-  my $grammar = MarpaX::Languages::C::AST::Grammar->new($grammarName, \%logInfo, $start);
+  my $grammar = MarpaX::Languages::C::AST::Grammar->new($grammarName, \%logInfo, $opts{start}, $opts{actionObject}, $opts{nonTerminalSemantic}, $opts{terminalSemantic});
   my $grammar_option = $grammar->grammar_option();
   $grammar_option->{bless_package} = 'C::AST';
   $grammar_option->{source} = \$grammar->content();
@@ -79,7 +78,7 @@ sub new {
 	       _typedef            => $typedef,
 	       _enum               => $enum,
 	       _lazy               => $lazy,
-	       _start              => $start
+	       _start              => $opts{start}
               };
 
   bless($self, $class);
@@ -711,7 +710,7 @@ __END__
 
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
@@ -719,7 +718,7 @@ MarpaX::Languages::C::AST - Translate a C source to an AST
 
 =head1 VERSION
 
-version 0.36
+version 0.40
 
 =head1 SYNOPSIS
 
@@ -791,6 +790,18 @@ A string giving the starting point of the grammar. This should be used when you
 
 Please note that giving another value but 'translationUnit' will emit warnings from the grammar, saying that some rules are not reachable.
 
+=item actionObject
+
+Grammar specific action object.
+
+=item nonTerminalSemantic
+
+Grammar specific non-terminal semantic action.
+
+=item terminalSemantic
+
+Grammar specific terminal semantic action.
+
 =item logInfo
 
 Reference to an array of lexemes for which a log of level INFO will be issued.
@@ -878,10 +889,16 @@ Jean-Damien Durand <jeandamiendurand@free.fr>
 
 =head1 CONTRIBUTORS
 
+=for stopwords Ben Bullock Jeffrey Kegler jddurand
+
 =over 4
 
 =item *
 
+Ben Bullock <benkasminbullock@gmail.com>
+
+=item *
+
 Jeffrey Kegler <JKEGL@cpan.org>
 
 =item *
