@@ -8,7 +8,7 @@ use IO::String;
 
 # ABSTRACT: ISO ANSI C 2011 grammar written in Marpa BNF
 
-our $VERSION = '0.36'; # TRIAL VERSION
+our $VERSION = '0.40'; # TRIAL VERSION
 
 
 our %DEFAULT_PAUSE = (
@@ -26,14 +26,20 @@ our %DEFAULT_PAUSE = (
     ANY_ASM              => 'after',
 );
 
+our $DEFAULTACTIONOBJECT = sprintf('%s::%s', __PACKAGE__, 'Actions');
+our $DEFAULTNONTERMINALSEMANTIC = ':default ::= action => [values] bless => ::lhs';
+our $DEFAULTTERMINALSEMANTIC = 'lexeme default = action => [start,length,value] forgiving => 1';
+
 our $DATA = do {local $/; <DATA>};
 
 sub new {
-  my ($class, $pausep, $start) = @_;
+  my ($class, $pausep, $start, $actionObject, $nonTerminalSemantic, $terminalSemantic) = @_;
+
+  $actionObject //= $DEFAULTACTIONOBJECT;
 
   my $self  = {
-    _grammar_option => {action_object  => sprintf('%s::%s', __PACKAGE__, 'Actions')},
-    _recce_option => {ranking_method => 'high_rule_only'},
+    _grammar_option => {},
+    _recce_option => {semantics_package => $actionObject, ranking_method => 'high_rule_only'},
   };
   #
   # Rework the grammar to have the pauses:
@@ -51,7 +57,15 @@ sub new {
 
   $self->{_content} = '';
   my $allb = exists($pause{__ALL__});
-  $start ||= 'translationUnit';
+  my $pragmas = '';
+  if (defined($start) && "$start") {
+    #
+    # User gave a custom start, we assume he will hit inaccessible symbols
+    #
+    $pragmas = "\ninaccessible is ok by default\n";
+  } else {
+    $start = 'translationUnit';
+  }
   my $data = IO::String->new($DATA);
   while (defined($_ = <$data>)) {
       my $line = $_;
@@ -76,8 +90,13 @@ sub new {
       }
       $self->{_content} .= $line;
   }
+  $nonTerminalSemantic //= $DEFAULTNONTERMINALSEMANTIC;
+  $terminalSemantic //= $DEFAULTTERMINALSEMANTIC;
 
+  $self->{_content} =~ s/\$PRAGMAS\n/$pragmas/;
   $self->{_content} =~ s/\$START\n/$start/;
+  $self->{_content} =~ s/\$NONTERMINALSEMANTIC\b/$nonTerminalSemantic/;
+  $self->{_content} =~ s/\$TERMINALSEMANTIC\b/$terminalSemantic/;
 
   bless($self, $class);
 
@@ -106,7 +125,7 @@ sub recce_option {
 
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
@@ -114,7 +133,7 @@ MarpaX::Languages::C::AST::Grammar::ISO_ANSI_C_2011 - ISO ANSI C 2011 grammar wr
 
 =head1 VERSION
 
-version 0.36
+version 0.40
 
 =head1 SYNOPSIS
 
@@ -134,9 +153,9 @@ This modules contains the ISO ANSI C 2011 C grammar written in Marpa BNF, as of
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new([$pausep, $start])
+=head2 new([$pausep, $start, $actionObject, $nonTerminalSemantic, $terminalSemantic])
 
-Instance a new object. Takes an eventual reference to a HASH for lexemes for which a pause after is requested, followed by an eventual start rule. Default paused lexemes is hardcoded to a list of lexeme that must always be paused, and this list cannot be altered. Default start rule is 'translationUnit'.
+Instance a new object. Takes an eventual reference to a HASH for lexemes for which a pause after is requested, followed by an eventual start rule, an eventual action object, an eventual default non-terminal semantic action, a default terminal semantic action. Default paused lexemes is hardcoded to a list of lexeme that must always be paused, and this list cannot be altered. Default start rule is 'translationUnit'. Default action object is hardcoded to __PACKAGE__::Actions module. Default non-terminal semantic is hardcoded to ':default ::= action => [values] bless => ::lhs'. Default terminal semantic is hardcoded to :'lexeme default = action => [start,length,value] forgiving => 1'.
 
 =head2 content()
 
@@ -171,12 +190,12 @@ __DATA__
 #                   http://www.quut.com/c/ANSI-C-grammar-y-2011.html
 #
 ################################################################################################################
-
+$PRAGMAS
 #
 # Defaults
 #
-:default ::= action => [values] bless => ::lhs
-lexeme default = action => [start,length,value] forgiving => 1
+$NONTERMINALSEMANTIC
+$TERMINALSEMANTIC
 
 #
 # G1 (grammar), c.f. http://www.quut.com/c/ANSI-C-grammar-y-2011.html
@@ -213,9 +232,7 @@ string
 genericSelection
 	::= GENERIC LPAREN assignmentExpression COMMA genericAssocList RPAREN
 
-genericAssocList
-	::= genericAssociation
-	| genericAssocList COMMA genericAssociation
+genericAssocList ::= genericAssociation+ separator => COMMA proper => 1
 
 genericAssociation
 	::= typeName COLON assignmentExpression
@@ -420,9 +437,7 @@ declarationSpecifiers2 ::=       # List with one or more typeSpecifier2
 
 # declarationSpecifiers ::= declarationSpecifiersUnit+
 
-initDeclaratorList
-	::= initDeclarator
-	| initDeclaratorList COMMA initDeclarator
+initDeclaratorList ::= initDeclarator+ separator => COMMA proper => 1
 
 initDeclarator
 	::= declarator EQUAL initializer
@@ -475,8 +490,8 @@ event 'structContextEnd[]' = nulled <structContextEnd>
 structContextEnd ::=
 
 structOrUnionSpecifier
-	::= structOrUnion LCURLY <structContextStart> structDeclarationList RCURLY <structContextEnd>
-	| structOrUnion IDENTIFIER_UNAMBIGUOUS LCURLY <structContextStart> structDeclarationList RCURLY <structContextEnd>
+	::= structOrUnion LCURLY (<structContextStart>) structDeclarationList RCURLY (<structContextEnd>)
+	| structOrUnion IDENTIFIER_UNAMBIGUOUS LCURLY (<structContextStart>) structDeclarationList RCURLY (<structContextEnd>)
 	| structOrUnion IDENTIFIER_UNAMBIGUOUS
 
 structOrUnion
@@ -519,9 +534,7 @@ specifierQualifierList2 ::= # List with one or more typeSpecifier2
                           | specifierQualifierList2 typeQualifier
                           | specifierQualifierList2 gccExtension
 
-structDeclaratorList
-	::= structDeclarator
-	| structDeclaratorList COMMA structDeclarator
+structDeclaratorList ::= structDeclarator+ separator => COMMA proper => 1
 
 structDeclarator
 	::= COLON constantExpression
@@ -530,14 +543,13 @@ structDeclarator
 
 enumSpecifier
 	::= ENUM LCURLY enumeratorList RCURLY
-	| ENUM LCURLY enumeratorList COMMA RCURLY
 	| ENUM IDENTIFIER_UNAMBIGUOUS LCURLY enumeratorList RCURLY
-	| ENUM IDENTIFIER_UNAMBIGUOUS LCURLY enumeratorList COMMA RCURLY
 	| ENUM IDENTIFIER_UNAMBIGUOUS
 
-enumeratorList
-	::= enumerator
-	| enumeratorList COMMA enumerator
+#
+# Saying 0 allow to have a final COMMA after the list
+#
+enumeratorList ::= enumerator+ separator => COMMA proper => 0
 
 enumerator	# identifiers must be flagged as ENUMERATION_CONSTANT
 	::= enumerationConstant EQUAL constantExpression
@@ -619,24 +631,23 @@ parameterTypeList
 	::= parameterList COMMA ELLIPSIS
 	| parameterList
 
-parameterList
-	::= parameterDeclaration
-	| parameterList COMMA parameterDeclaration
+parameterList ::= parameterDeclaration+ separator => COMMA proper => 1
 
 event 'parameterDeclarationdeclarationSpecifiers$' = completed <parameterDeclarationdeclarationSpecifiers>
 parameterDeclarationdeclarationSpecifiers ::= declarationSpecifiers
 
 event 'parameterDeclarationCheck$' = completed <parameterDeclarationCheck>
-parameterDeclarationCheck ::= parameterDeclarationdeclarationSpecifiers declarator
+parameterDeclarationCheck ::= parameterDeclarationdeclarationSpecifiers parameterDeclarationCheckDeclarator
+
+event 'parameterDeclarationCheckDeclarator$' = completed <parameterDeclarationCheckDeclarator>
+parameterDeclarationCheckDeclarator ::= declarator
 
 parameterDeclaration
 	::= parameterDeclarationCheck               rank =>  0
 	| declarationSpecifiers abstractDeclarator  rank => -1
 	| declarationSpecifiers                     rank => -2
 
-identifierList
-	::= IDENTIFIER
-	| identifierList COMMA IDENTIFIER
+identifierList ::= IDENTIFIER+ separator => COMMA proper => 1
 
 typeName
 	::= specifierQualifierList abstractDeclarator
@@ -710,6 +721,7 @@ statement
 labeledStatement
 	::= IDENTIFIER COLON statement
 	| CASE constantExpression COLON statement
+	| CASE constantExpression (WS_MANY) ELLIPSIS (WS_MANY) constantExpression COLON statement
 	| DEFAULT COLON statement
 
 compoundStatement
@@ -747,12 +759,13 @@ jumpStatement
 	| RETURN expression SEMICOLON
 
 event 'translationUnit$' = completed <translationUnit>
-translationUnit ::= externalDeclaration+
+translationUnit ::= externalDeclaration*
 
 event '^externalDeclaration' = predicted <externalDeclaration>
 externalDeclaration
 	::= functionDefinition
 	| declaration
+        | (SEMICOLON)
 
 compoundStatementReenterScope ::= LCURLY_REENTERSCOPE RCURLY_SCOPE
 	                        | LCURLY_REENTERSCOPE blockItemList RCURLY_SCOPE
@@ -827,7 +840,7 @@ CP_maybe   ~
 SP         ~ 'u8' | [uUL]
 SP_maybe   ~ SP
 SP_maybe   ~
-ES_AFTERBS ~ [\'\"\?\\abfnrtv]
+ES_AFTERBS ~ [\'\"\?\\abfnrtve]
            | O
            | O O
            | O O O
@@ -836,6 +849,7 @@ ES         ~ BS ES_AFTERBS
 WS         ~ [ \t\v\n\f]
 WS_any     ~ WS*
 WS_many    ~ WS+
+WS_MANY    ~ WS+
 
 # Lexemes
 :lexeme ~ <AUTO>          priority => -1
